@@ -24,12 +24,9 @@
   #:use-module (nyacc lang util)
   )
 
-;;(include "tables.scm")
-(include-from-path "nyacc/lang/javascript/tables.scm")
-;;(include "pbody.scm")
-(include-from-path "nyacc/lang/javascript/pbody.scm")
-;;(include "actions.scm")
-(include-from-path "nyacc/lang/javascript/actions.scm")
+(include-from-path "nyacc/lang/javascript/mach.d/jstab.scm")
+(include-from-path "nyacc/lang/javascript/body.scm")
+(include-from-path "nyacc/lang/javascript/mach.d/jsact.scm")
 
 ;; Parse given a token generator.  Uses fluid @code{*info*}.
 (define raw-parser
