@@ -42,7 +42,7 @@
   #:use-module (nyacc util)
   )
 
-(define *nyacc-version* "0.78.3")
+(define *nyacc-version* "0.79.0")
 
 ;; @deffn proxy-? sym rhs
 ;; @example
@@ -113,7 +113,6 @@
 ;; @item @code{('terminal . atom)}
 ;; @item @code{('non-terminal . symbol)}
 ;; @item @code{('action . (ref narg guts)}
-;; @item @code{('proxy . production-rule)}
 ;; @end itemize
 ;; Currently, the number of arguments for items is computed in the routine
 ;; @code{process-grammar}.
@@ -121,116 +120,108 @@
 (define-syntax lalr-spec
   (syntax-rules +++ () 
     ((_ <expr> +++)
-     (let* ()
-       (letrec-syntax
-	   ((with-attr-list
-	     (syntax-rules ($prune)
-	       ((_ ($prune <symb>) <ex> ...)
-		(cons '(prune . <symb>) (with-attr-list <ex> ...)))
-	       ((_) '())))
-	    (parse-rhs
-	     (lambda (x)
-	       ;; The following is syntax-case because we use a fender.
-	       (syntax-case x (quote $$ $$/ref $$-ref $prec $with $empty
-				     $? $* $+)
-		 ;; action specifications
-		 ((_ ($$ <guts> ...) <e2> ...)
-		  #'(cons '(action #f #f <guts> ...) (parse-rhs <e2> ...)))
-		 ((_ ($$-ref <ref>) <e2> ...)
-		  ;;#'(cons '(action #f <ref> #f) (parse-rhs <e2> ...)))
-		  #'(cons `(action #f ,<ref> . #f) (parse-rhs <e2> ...)))
-		 ((_ ($$/ref <ref> <guts> ...) <e2> ...)
-		  #'(cons `(action #f ,<ref> <guts> ...) (parse-rhs <e2> ...)))
+     (letrec-syntax
+	 ((parse-rhs
+	  (lambda (x)
+	    ;; The following is syntax-case because we use a fender.
+	    (syntax-case x (quote $$ $$/ref $$-ref $prec $empty $? $* $+)
+	      ;; action specifications
+	      ((_ ($$ <guts> ...) <e2> ...)
+	       #'(cons '(action #f #f <guts> ...) (parse-rhs <e2> ...)))
+	      ((_ ($$-ref <ref>) <e2> ...)
+	       ;;#'(cons '(action #f <ref> #f) (parse-rhs <e2> ...)))
+	       #'(cons `(action #f ,<ref> . #f) (parse-rhs <e2> ...)))
+	      ((_ ($$/ref <ref> <guts> ...) <e2> ...)
+	       #'(cons `(action #f ,<ref> <guts> ...) (parse-rhs <e2> ...)))
 
-		 ;; other internal $-syntax
-		 ((_ ($prec <tok>) <e2> ...)
-		  #'(cons (cons 'prec (tokenize <tok>)) (parse-rhs <e2> ...)))
-		 ((_ ($with <lhs-ref> <ex> ...) <e2> ...)
-		  #'(cons `(with <lhs-ref> ,@(with-attr-list <ex> ...))
-			  (parse-rhs <e2> ...)))
-		 ((_ $empty <e2> ...)	; TODO: propagate to processor
-		  #'(parse-rhs <e2> ...))
-		 
-		 ;; (experimental) proxies
-		 ((_ ($? <s1> <s2> ...) <e2> ...)
-		  #'(cons (cons* 'proxy proxy-? (parse-rhs <s1> <s2> ...))
-			  (parse-rhs <e2> ...)))
-		 ((_ ($+ <s1> <s2> ...) <e2> ...)
-		  #'(cons (cons* 'proxy proxy-+ (parse-rhs <s1> <s2> ...))
-			  (parse-rhs <e2> ...)))
-		 ((_ ($* <s1> <s2> ...) <e2> ...)
-		  #'(cons (cons* 'proxy proxy-* (parse-rhs <s1> <s2> ...))
-			  (parse-rhs <e2> ...)))
-		 
-		 ;; terminals and non-terminals
-		 ((_ (quote <e1>) <e2> ...)
-		  #'(cons '(terminal . <e1>) (parse-rhs <e2> ...)))
-		 ((_ (<f> ...) <e2> ...)
-		  #'(cons (<f> ...) (parse-rhs <e2> ...)))
-		 ((_ <e1> <e2> ...)
-		  (identifier? (syntax <e1>)) ; fender to trap non-term's
-		  (if (reserved? (syntax <e1>))
-		      #'(cons '(terminal . <e1>) (parse-rhs <e2> ...))
-		      #'(cons '(non-terminal . <e1>) (parse-rhs <e2> ...))))
-		 ((_ <e1> <e2> ...)
-		  #'(cons '(terminal . <e1>) (parse-rhs <e2> ...)))
-		 ((_) #'(list)))))
-	    (parse-rhs-list
-	     (syntax-rules ()
-	       ((_ (<ex> ...) <rhs> ...)
-		(cons (parse-rhs <ex> ...)
-		      (parse-rhs-list <rhs> ...)))
-	       ((_) '())))
-	    (parse-grammar
-	     (syntax-rules ()
-	       ((_ (<lhs> <rhs> ...) <prod> ...)
-		(cons (cons '<lhs> (parse-rhs-list <rhs> ...))
-		      (parse-grammar <prod> ...)))
-	       ((_) '())))
-	    (tokenize
-	     (lambda (x)
-	       (syntax-case x ()
-		 ((_ <tk>) (identifier? (syntax <tk>)) #'(quote <tk>))
-		 ((_ <tk>) #'<tk>))))
-	    (tokenize-list
-	     (syntax-rules ()
-	       ((_ <tk1> <tk2> ...)
-		(cons (tokenize <tk1>) (tokenize-list <tk2> ...)))
-	       ((_) '())))
-	    (parse-precedence
-	     (syntax-rules (left right nonassoc)
-	       ((_ (left <tk> ...) <ex> ...)
-		(cons (cons 'left (tokenize-list <tk> ...))
-		      (parse-precedence <ex> ...)))
-	       ((_ (right <tk> ...) <ex> ...)
-		(cons (cons 'right (tokenize-list <tk> ...))
-		      (parse-precedence <ex> ...)))
-	       ((_ (nonassoc <tk> ...) <ex> ...)
-		(cons (cons 'nonassoc (tokenize-list <tk> ...))
-		      (parse-precedence <ex> ...)))
-	       ((_ <tk> <ex> ...)
-		(cons (list 'undecl (tokenize <tk>))
-		      (parse-precedence <ex> ...)))
-	       ((_) '())))
-	    (lalr-spec-1
-	     (syntax-rules (start expect notice prec< prec> grammar)
-	       ((_ (start <symb>) <e> ...)
-		(cons (cons 'start '<symb>) (lalr-spec-1 <e> ...)))
-	       ((_ (expect <n>) <e> ...)
-		(cons (cons 'expect <n>) (lalr-spec-1 <e> ...)))
-	       ((_ (notice <str>) <e> ...)
-		(cons (cons 'notice <str>) (lalr-spec-1 <e> ...)))
-	       ((_ (prec< <ex> ...) <e> ...)
-		(cons (cons 'precedence (parse-precedence <ex> ...))
-		      (lalr-spec-1 <e> ...)))
-	       ((_ (prec> <ex> ...) <e> ...)
-		(cons (cons 'precedence (reverse (parse-precedence <ex> ...)))
-		      (lalr-spec-1 <e> ...)))
-	       ((_ (grammar <prod> ...) <e> ...)
-		(cons (cons 'grammar (parse-grammar <prod> ...))
-		      (lalr-spec-1 <e> ...))) 
-	       ((_) '()))))
-	 (process-spec (lalr-spec-1 <expr> +++)))))))
+	      ;; other internal $-syntax
+	      ((_ ($prec <tok>) <e2> ...)
+	       #'(cons (cons 'prec (tokenize <tok>)) (parse-rhs <e2> ...)))
+	      ((_ $empty <e2> ...)	; TODO: propagate to processor
+	       #'(parse-rhs <e2> ...))
+	      
+	      ;; (experimental) proxies
+	      ((_ ($? <s1> <s2> ...) <e2> ...)
+	       #'(cons (cons* 'proxy proxy-? (parse-rhs <s1> <s2> ...))
+		       (parse-rhs <e2> ...)))
+	      ((_ ($+ <s1> <s2> ...) <e2> ...)
+	       #'(cons (cons* 'proxy proxy-+ (parse-rhs <s1> <s2> ...))
+		       (parse-rhs <e2> ...)))
+	      ((_ ($* <s1> <s2> ...) <e2> ...)
+	       #'(cons (cons* 'proxy proxy-* (parse-rhs <s1> <s2> ...))
+		       (parse-rhs <e2> ...)))
+	      
+	      ;; terminals and non-terminals
+	      ((_ (quote <e1>) <e2> ...)
+	       #'(cons '(terminal . <e1>) (parse-rhs <e2> ...)))
+	      ((_ (<f> ...) <e2> ...)
+	       #'(cons (<f> ...) (parse-rhs <e2> ...)))
+	      ((_ <e1> <e2> ...)
+	       (identifier? (syntax <e1>)) ; fender to trap non-term's
+	       (if (reserved? (syntax <e1>))
+		   #'(cons '(terminal . <e1>) (parse-rhs <e2> ...))
+		   #'(cons '(non-terminal . <e1>) (parse-rhs <e2> ...))))
+	      ((_ <e1> <e2> ...)
+	       #'(cons '(terminal . <e1>) (parse-rhs <e2> ...)))
+	      ((_) #'(list)))))
+       (parse-rhs-list
+	(syntax-rules ()
+	  ((_ (<ex> ...) <rhs> ...)
+	   (cons (parse-rhs <ex> ...)
+		 (parse-rhs-list <rhs> ...)))
+	  ((_) '())))
+       (parse-grammar
+	(syntax-rules ()
+	  ((_ (<lhs> <rhs> ...) <prod> ...)
+	   (cons (cons '<lhs> (parse-rhs-list <rhs> ...))
+		 (parse-grammar <prod> ...)))
+	  ((_) '())))
+       (tokenize
+	(lambda (x)
+	  (syntax-case x ()
+	    ((_ <tk>) (identifier? (syntax <tk>)) #'(quote <tk>))
+	    ((_ <tk>) #'<tk>))))
+       (tokenize-list
+	(syntax-rules ()
+	  ((_ <tk1> <tk2> ...)
+	   (cons (tokenize <tk1>) (tokenize-list <tk2> ...)))
+	  ((_) '())))
+       (parse-precedence
+	(syntax-rules (left right nonassoc)
+	  ((_ (left <tk> ...) <ex> ...)
+	   (cons (cons 'left (tokenize-list <tk> ...))
+		 (parse-precedence <ex> ...)))
+	  ((_ (right <tk> ...) <ex> ...)
+	   (cons (cons 'right (tokenize-list <tk> ...))
+		 (parse-precedence <ex> ...)))
+	  ((_ (nonassoc <tk> ...) <ex> ...)
+	   (cons (cons 'nonassoc (tokenize-list <tk> ...))
+		 (parse-precedence <ex> ...)))
+	  ((_ <tk> <ex> ...)
+	   (cons (list 'undecl (tokenize <tk>))
+		 (parse-precedence <ex> ...)))
+	  ((_) '())))
+       (lalr-spec-1
+	(syntax-rules (start expect notice reserve prec< prec> grammar)
+	  ((_ (start <symb>) <e> ...)
+	   (cons (cons 'start '<symb>) (lalr-spec-1 <e> ...)))
+	  ((_ (expect <n>) <e> ...)
+	   (cons (cons 'expect <n>) (lalr-spec-1 <e> ...)))
+	  ((_ (notice <str>) <e> ...)
+	   (cons (cons 'notice <str>) (lalr-spec-1 <e> ...)))
+	  ((_ (reserve <t1> ...) <e> ...)
+	   (cons (list 'reserve <t1> ...) (lalr-spec-1 <e> ...)))
+	  ((_ (prec< <ex> ...) <e> ...)
+	   (cons (cons 'precedence (parse-precedence <ex> ...))
+		 (lalr-spec-1 <e> ...)))
+	  ((_ (prec> <ex> ...) <e> ...)
+	   (cons (cons 'precedence (reverse (parse-precedence <ex> ...)))
+		 (lalr-spec-1 <e> ...)))
+	  ((_ (grammar <prod> ...) <e> ...)
+	   (cons (cons 'grammar (parse-grammar <prod> ...))
+		 (lalr-spec-1 <e> ...))) 
+	  ((_) '()))))
+       (process-spec (lalr-spec-1 <expr> +++))))))
 
 ;; @deffn atomize terminal => object
 ;; Generate an atomic object for a terminal.   Expected terminals are strings,
@@ -359,7 +350,7 @@
 		 (cdr all))))))
 
   ;; TODO: check for repeated tokens in precedence spec's: prec<, prec>
-	       
+
   (let* ((gram (assq-ref tree 'grammar))
 	 (start-symbol (and=> (assq-ref tree 'start) atomize))
 	 (start-rule (lambda () (list start-symbol)))
@@ -379,7 +370,8 @@
 	       (@l (list		; attributes per prod' rule
 		    `((rhs . ,(vector start-symbol))
 		      (ref . all) (act 1 $1))))
-	       (tl '($code-comm $lone-comm $error $end)) ; set of terminals
+	       (tl (append '($code-comm $lone-comm $error $end) ; terminals
+			   (or (assq-ref tree 'reserve) '())))
 	       (nl (list start-symbol))	; set of non-terminals
 	       ;;
 	       (head gram)	       ; head of unprocessed productions
@@ -419,14 +411,6 @@
 	  ((prec)
 	   (iter ll @l (add-el (cdar rhs) tl) nl head prox lhs tail rhs-l
 		 (acons 'prec (atomize (cdar rhs)) attr) pel (cdr rhs)))
-	  ((with)
-	   (let* ((psy (maksy))		      ; proxy symbol
-		  (rhsx (cadar rhs))	      ; symbol to expand
-		  (p-l (map cdr (cddar rhs))) ; prune list
-		  (p1 (list psy `((non-terminal . ,rhsx)
-				  (action #f #f $1)))))
-	     (iter ll @l tl (cons psy nl) head (cons p1 prox) lhs tail rhs-l
-		   (acons 'with (cons psy p-l) attr) (cons psy pel) (cdr rhs))))
 	  (else
 	   (error (fmtstr "bug=~S" (caar rhs))))))
 
@@ -440,7 +424,6 @@
 	;; @end itemize
 	(let* ((ln (length pel))
 	       (action (assq-ref attr 'action))
-	       (with (assq-ref attr 'with))
 	       (nrg (if action (or (car action) ln) ln))  ; number of args
 	       (ref (if action (cadr action) #f))
 	       (act (cond
@@ -449,7 +432,6 @@
 		     ((memq '$error pel) '((display "syntax error\n")))
 		     ((zero? nrg) '((list)))
 		     (else '($1)))))
-	  (if with (simple-format #t "WITH WHAT?\n"))
 	  (iter (cons lhs ll)
 		(cons (cons* (cons 'rhs (list->vector (reverse pel)))
 			     (cons* 'act nrg act) (cons 'ref ref) attr) @l)
@@ -484,15 +466,14 @@
 
        (else
 	(let* ((al (reverse @l))	; attribute list
-	       (err-1 '()) ;; not used
+	       (err-l '())
 	       ;; symbol used as terminal and non-terminal:
-	       (err-2 (gram-check-2 tl nl err-1))
+	       (err-l (gram-check-2 tl nl err-l))
 	       ;; non-terminal's w/o production rule:
-	       (err-3 (gram-check-3 ll nl err-2))
+	       (err-l (gram-check-3 ll nl err-l))
 	       ;; TODO: which don't appear in OTHER RHS, e.g., (foo (foo))
-	       (err-4 (gram-check-4 ll nl err-3))
-	       ;; todo: Check that with withs are not mixed
-	       (err-l err-4))
+	       (err-l (gram-check-4 ll nl err-l))
+	       )
 	  (for-each (lambda (e) (fmterr "~A\n" e)) err-l)
 	  (if (pair? (filter (lambda (s) (char=? #\* (string-ref s 0))) err-l))
 	      #f
@@ -1533,20 +1514,22 @@
 ;;   kis-v - itemsets
 ;;   pat-v - action table
 
-;; @deffn restart-spec [spec|mach] start => spec
+;; @deffn {Procedure} restart-spec [spec|mach] start => spec
 ;; This generates a new spec with a different start.
 ;; @example
 ;; (restart-spec clang-spec 'expression) => cexpr-spec
 ;; @end example
+;; @end deffn
 (define (restart-spec spec start)
   (let* ((rhs-v (vector-copy (assq-ref spec 'rhs-v))))
     (vector-set! rhs-v 0 (vector start))
     (cons* (cons 'rhs-v rhs-v)
 	   (member '(restart-tail . #t) spec))))
 
-;; @deffn make-lalr-machine spec => pgen
+;; @deffn {Procedure} make-lalr-machine spec => pgen
 ;; Generate a-list of items used for building/debugging parsers.
 ;; It might be useful to add hashify and compact with keyword arguments.
+;; @end deffn
 (define (make-lalr-machine spec)
   (if (not spec) (error "make-lalr-machine: expecting valid specification"))
   (let ((prev-core (fluid-ref *lalr-core*)))
@@ -1589,9 +1572,10 @@
 (define (lalr-match-table mach)
   (assq-ref mach 'mtab))
 
-;; @deffn machine-compacted? mach => #t|#f
+;; @deffn {Procedure} machine-compacted? mach => #t|#f
 ;; Indicate if the machine has been compacted.
 ;; TODO: needs update to deal with error recovery hooks.
+;; @end deffn
 (define (machine-compacted? mach)
   ;; Works by searching for $default phony-token.
   (call-with-prompt 'got-it
@@ -1612,7 +1596,7 @@
 ;; See @code{compact-machine} below.
 (define default-keepers '($error $lone-comm $code-comm $end))
 
-;; @deffn compact-machine mach [#:keep 3] [#:keepers '()] => mach
+;; @deffn {Procedure} compact-machine mach [#:keep 3] [#:keepers '()] => mach
 ;; A "filter" to compact the parse table.  For each state this will replace
 ;; the most populus set of reductions of the same production rule with a
 ;; default production.  However, reductions triggered by user-specified keepers
@@ -1726,13 +1710,15 @@
 
 ;; NEW: need to add reduction of ERROR
 
-;; @deffn machine-hashed? mach => #t|#f
+;; @deffn {Procedure} machine-hashed? mach => #t|#f
 ;; Indicate if the machine has been hashed.
+;; @end deffn
 (define (machine-hashed? mach)
   ;; If hashed, the parse action for rule 0 will always be a number.
   (number? (caar (vector-ref (assq-ref mach 'pat-v) 0))))
 
-;; @deffn hashify-machine mach => mach
+;; @deffn {Procedure} hashify-machine mach => mach
+;; @end deffn
 (define (hashify-machine mach)
   (if (machine-hashed? mach) mach
       (let* ((terminals (assq-ref mach 'terminals))
